import 'dart:async';
import 'dart:io';

import 'package:adpluginx/Authentication/AppAuth.dart';
import 'package:adpluginx/Model/UserModel.dart';
import 'package:adpluginx/Subsciption/InAppStore.dart';
import 'package:adpluginx/Subsciption/ManageSub.dart';
import 'package:flutter/material.dart';
import 'package:in_app_purchase/in_app_purchase.dart';
import 'package:in_app_purchase_android/in_app_purchase_android.dart';

class IapProvider extends ChangeNotifier {
  final InAppPurchase inAppPurchase = InAppPurchase.instance;
  StreamSubscription<List<PurchaseDetails>>? subscription;
  List<String> notFoundIds = <String>[];
  List<ProductDetails> products = <ProductDetails>[];
  final List<PurchaseDetails> purchases = <PurchaseDetails>[];
  bool isAvailable = false;
  bool purchasePending = false;
  bool loading = true;
  bool purchasing = true;
  bool isSubscribed = false;
  Map planData = {};


  List<ProductDetails> getSortedPurchases() {
    products.sort((a, b) => a.rawPrice.compareTo(b.rawPrice),);
    return products;
  }

  setSub(bool state) {
    isSubscribed = state;
    notifyListeners();
  }

  setPlan(Map state) {
    planData = state;
    notifyListeners();
  }

  void init() {
    final Stream<List<PurchaseDetails>> purchaseUpdated =
        inAppPurchase.purchaseStream;
    subscription =
        purchaseUpdated.listen((List<PurchaseDetails> purchaseDetailsList) {
          listenToPurchaseUpdated(purchaseDetailsList);
          checkSubscription(purchaseDetailsList);
        }, onDone: () {
          subscription?.cancel();
        }, onError: (Object error) {
          print(error);
        });
    establishConnection();
  }

  void dispose() {
    subscription?.cancel();
  }

  List userPurchaseSortedList = [];

  findLastSubscription() {
    List buffer = List.from(userPurchases.toList());
    buffer.sort((a, b) {
      DateTime aDate = DateTime.fromMillisecondsSinceEpoch(
          int.parse(b.transactionDate ?? "0"));
      DateTime bDate = DateTime.fromMillisecondsSinceEpoch(
          int.parse(a.transactionDate ?? "0"));
      return aDate.compareTo(bDate);
    });
    userPurchaseSortedList = buffer;
    notifyListeners();
  }

  Future<void> establishConnection() async {
    UserModel? userModel = await AppAuthProvider.getPrefUser();

    final bool isConnected = await inAppPurchase.isAvailable();
    if (isConnected) {
      isAvailable = isConnected;
      notifyListeners();
      getProducts();
      inAppPurchase.restorePurchases(
        applicationUserName: userModel?.appleIdCode,
      );
    }
  }

  Future<void> getProducts() async {
    final ProductDetailsResponse productDetailResponse =
    await inAppPurchase.queryProductDetails(productIds.toSet());
    if (productDetailResponse.productDetails.isNotEmpty) {
      products = productDetailResponse.productDetails;
      notFoundIds = productDetailResponse.notFoundIDs;
      purchasePending = false;
      loading = true;
      notifyListeners();
    } else {
      purchasePending = false;
      loading = false;
      notifyListeners();
    }
  }

  Future<void> handlePurchase(ProductDetails productDetails) async {
    UserModel? userModel = await AppAuthProvider.getPrefUser();
    late PurchaseParam purchaseParam;
    Platform.isAndroid
        ? purchaseParam = GooglePlayPurchaseParam(
      productDetails: productDetails,
      applicationUserName: userModel?.appleIdCode,
    )
        : purchaseParam = PurchaseParam(
      productDetails: productDetails,
      applicationUserName: userModel?.appleIdCode,
    );

    switch (productDetails.id) {
      default:
        inAppPurchase.buyNonConsumable(purchaseParam: purchaseParam);
        break;
    }
  }

  Future<void> deliverProduct(PurchaseDetails purchaseDetails) async {
    print(purchaseDetails.productID);
    if (purchaseDetails.productID == weeklySubscriptionId) {
      // ManageSubscription.updateWeeklySub(purchaseDetails);
    } else if (purchaseDetails.productID == monthlySubscriptionId) {
      // ManageSubscription.updateMonthlySub(purchaseDetails);
    }
    purchasePending = false;
    purchases.add(purchaseDetails);
    notifyListeners();
  }

  void handleError(IAPError error) {
    // ignore: avoid_print
    print("Error caught: $error");
    purchasePending = false;
    notifyListeners();
  }

  void _handleInvalidPurchase(PurchaseDetails purchaseDetails) {
    // ignore: avoid_print
    print("There was error purchase productId ${purchaseDetails.productID}");
  }

  void checkSubscription(List<PurchaseDetails> purchaseDetailsList) {
    if (purchaseDetailsList.isNotEmpty) {
      ManageSub().setSub(purchaseDetailsList);
    } else {
      ManageSub().resetSub();
    }
  }

  Set<PurchaseDetails> userPurchases = {};

  Future<void> listenToPurchaseUpdated(
      List<PurchaseDetails> purchaseDetailsList) async {
    checkSubscription(purchaseDetailsList);

    for (final PurchaseDetails purchaseDetails in purchaseDetailsList) {
      userPurchases.add(purchaseDetails);

      if (purchaseDetails.status == PurchaseStatus.pending) {
        purchasePending = true;
        notifyListeners();
      } else {
        if (purchaseDetails.status == PurchaseStatus.error) {
          handleError(purchaseDetails.error!);
        } else if (purchaseDetails.status == PurchaseStatus.purchased ||
            purchaseDetails.status == PurchaseStatus.restored) {
          if (purchaseDetails.pendingCompletePurchase) {
            await inAppPurchase.completePurchase(purchaseDetails);
          }
          deliverProduct(purchaseDetails);
        } else {
          _handleInvalidPurchase(purchaseDetails);
        }
      }
    }
    notifyListeners();
  }
}
