import 'dart:convert';

import 'package:adpluginx/adpluginx.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:in_app_purchase/in_app_purchase.dart';

class ManageSub {
  static final ManageSub _singleton = ManageSub._internal();

  factory ManageSub() {
    return _singleton;
  }

  ManageSub._internal();

  SharedPreferences? subPref;

  initSub() async {
    subPref = await SharedPreferences.getInstance();
  }

  setSub(List<PurchaseDetails> purchaseData) async {
    if (purchaseData.isEmpty) {
      return;
    }
    subPref ??= await SharedPreferences.getInstance();
    List buffer = List.from(purchaseData);
    buffer.sort((a, b) {
      DateTime aDate = DateTime.fromMillisecondsSinceEpoch(
          int.parse(b.transactionDate ?? "0"));
      DateTime bDate = DateTime.fromMillisecondsSinceEpoch(
          int.parse(a.transactionDate ?? "0"));
      return aDate.compareTo(bDate);
    });
    PurchaseDetails latest = buffer[0];
    if (latest.status == PurchaseStatus.restored ||
        latest.status == PurchaseStatus.purchased) {
      print("Calling =============> ${purchaseData.length}");
      int p = int.parse(latest.productID.toString().split("_days_")[0] ?? "0");
      DateTime purchaseDate = DateTime.fromMillisecondsSinceEpoch(
          int.parse(latest.transactionDate ?? "0"));
      // TAodo Change To Orignal
      // DateTime endDate = purchaseDate.add(const Duration(minutes: 1));
      DateTime endDate = purchaseDate.add(Duration(days: p));
      print("PurchaseDate =============> $purchaseDate");
      print("Days =============> $p");
      print("EndDate =============> $endDate");
      if (endDate.isAfter(DateTime.now())) {
        print("Status =============> Subscribed");
        Map plan = {
          "ProductId": latest.productID,
          "PurchaseId": latest.purchaseID,
          "Status": latest.status.name,
          "PurchaseDate": latest.transactionDate,
        };
        subPref!.setBool(
          "subscription",
          true,
        );
        subPref!.setString(
          "plan",
          json.encode(plan),
        );
        Future.microtask(() {
          IapProvider iapProvider = NavigationService
              .navigatorKey.currentContext!
              .read<IapProvider>();
          iapProvider.setSub(true);
          iapProvider.setPlan(plan);
        });
      } else {
        print("Status =============> No SubScribe");
        resetSub();
      }
    } else {
      resetSub();
    }
  }

  resetSub() async {
    Future.microtask(() {
      IapProvider iapProvider =
          NavigationService.navigatorKey.currentContext!.read<IapProvider>();
      iapProvider.setSub(false);
      iapProvider.setPlan({});
    });
    subPref ??= await SharedPreferences.getInstance();
    subPref!.remove(
      "subscription",
    );
    subPref!.remove(
      "plan",
    );
  }

  Future<bool> isSub() async {
    subPref ??= await SharedPreferences.getInstance();
    return subPref!.getBool("subscription") ?? false;
  }

  Future<Map> getSubDetails() async {
    subPref ??= await SharedPreferences.getInstance();
    return json.decode(subPref!.getString("plan") ?? "{}");
  }
}
