// 🐦 Flutter imports:
import 'package:flutter/material.dart';

// 📦 Package imports:
import 'package:onesignal_flutter/onesignal_flutter.dart';
import 'package:provider/provider.dart';

// 🌎 Project imports:
import 'package:adpluginx/AdBase/AdBase.dart';
import 'package:adpluginx/Engine/AdEngine.dart';

extension AdState on BuildContext {
  Map? getMainJsonState({bool listen = true}) {
    return Provider.of<AdBase>(this, listen: listen).data;
  }
}

extension NavigationExtension on String {
  pushWithAds(BuildContext context) {
    AdEngine().showFullScreen(
      context: context,
      onJobComplete: () {
        Navigator.pushNamed(context, this);
      },
    );
  }

  pushReplaceWithAds(BuildContext context) {
    AdEngine().showFullScreen(
      context: context,
      onJobComplete: () {
        Navigator.pushReplacementNamed(context, this);
      },
    );
  }

  pushReplaceRemoveUntilWithAds(BuildContext context) {
    AdEngine().showFullScreen(
      context: context,
      onJobComplete: () {
        Navigator.pushNamedAndRemoveUntil(context, this, (route) => false);
      },
    );
  }

  performAction(
      {required BuildContext context, required Function() onComplete}) {
    AdEngine().showActionBasedAds(
      context: context,
      actionName: this,
      onJobComplete: () {
        onComplete();
      },
    );
  }

  initOnesignal() async {
    await OneSignal.shared.setAppId(this);
    OneSignal.shared.promptUserForPushNotificationPermission();
  }
}

// Concept For Key to Required
extension CheckKeys on Map? {
  checkKeys() {}
}

extension EmailValidator on String {
  bool isValidEmail() {
    return RegExp(
        r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$')
        .hasMatch(this);
  }
}
